#include <stdlib.h>
#include "wcstring.h"

void initstr(void *str, size_t n)
{
  memset(str, 0xff, n);
}

int main(void)
{
  wchar_t wcs1[8192], wcs2[8192], wcs3[8192];
  char str1[8192], str2[8192];

  /* ---------Testing for wcslen fucntion---------- */
  printf("wcslen() testing: ");

  /* test the empty string */  
  strtowcs(wcs1,"");
  if (wcslen(wcs1) == 0)
    printf("PASS, ");
  else
    printf("FAIL, ");

  /* test single character string */
  strtowcs(wcs1,"H");
  if (wcslen(wcs1) == 1)
    printf("PASS, ");
  else
    printf("FAIL, ");

  /* test multi-character string */
  strtowcs(wcs1, "Hello World");
  if (wcslen(wcs1) == 11)
    printf("PASS\n");
  else
    printf("FAIL\n");

  /* ---------Testing for wcscpy function---------- */
  /* Test the empty string */
  initstr(wcs1, sizeof (wcs1));
  initstr(wcs2, sizeof (wcs2));
  printf("wcscpy() testing: ");

  strtowcs(wcs1,"");
  wcscpy(wcs2, wcs1);
  if (memcmp(wcs1, wcs2, sizeof (wcs1)) == 0)
    printf ("PASS, ");
  else
    printf ("FAIL, ");
  
  /* Test 1 character string */
  initstr(wcs1, sizeof (wcs1));
  initstr(wcs2, sizeof (wcs2));
  strtowcs(wcs1,"H");
  wcscpy(wcs2, wcs1);
  if (memcmp(wcs1, wcs2, sizeof (wcs1)) == 0)
    printf ("PASS, ");
  else
    printf ("FAIL, ");

  /* Test multicharacter string */
  initstr(wcs1, sizeof (wcs1));
  initstr(wcs2, sizeof (wcs2));
  strtowcs(wcs1, "Hello World");
  wcscpy(wcs2, wcs1);
  if (memcmp(wcs1, wcs2, sizeof (wcs1)) == 0)
    printf ("PASS\n");
  else
    printf ("FAIL\n");

  

  /* ---------Testing for wcsncpy function---------- */
  printf("wcsncpy() testing: ");
  /* Test the empty string */
  memset(wcs1, 0, sizeof(wcs1));
  initstr(wcs2, sizeof (wcs2));

  strtowcs(wcs1,"");
  wcsncpy(wcs2, wcs1, 8191);
  if ( (memcmp(wcs1, wcs2, sizeof (wcs1)-4) == 0)&&(wcs2[8191] == 0xffffffff))
    printf ("PASS, ");
  else
    printf ("FAIL, ");
  
  /* Test 1 character string */
  initstr(wcs1, sizeof (wcs1));
  initstr(wcs2, sizeof (wcs2));
  strtowcs(wcs1,"H");
  wcsncpy(wcs2, wcs1, 8191);
  memset(wcs1,0x00,sizeof(wcs1));
  strtowcs(wcs1, "H");

  if (memcmp(wcs1, wcs2, sizeof (wcs1)-4) == 0&&(wcs2[8191] == 0xffffffff))
    printf ("PASS, ");
  else
    printf ("FAIL, ");

  /* Test multicharacter string */
  initstr(wcs1, sizeof (wcs1));
  initstr(wcs2, sizeof (wcs2));
  strtowcs(wcs1, "Hello World");
  wcsncpy(wcs2, wcs1, 8191);
  
  memset(wcs1, 0x00, sizeof(wcs1));
  strtowcs(wcs1, "Hello World");
  if ( (memcmp(wcs1, wcs2, sizeof (wcs1)-4) == 0)&&(wcs2[8191]==0xffffffff) )
    printf ("PASS, ");
  else
    printf ("FAIL, ");

  /* Test multicharacter string shorter than length of buffer */
  initstr(wcs2, sizeof(wcs2));
  wcsncpy(wcs2, wcs1,11);
  if(wcs2[11] == 0xffffffff && wcs2[10]== 'd')
    printf("PASS, ");
  else
    printf("FAIL, ");

  /* Test multicharacter string equal to lenght of buffer */
  initstr(wcs2, sizeof(wcs2));
  initstr(wcs1, sizeof(wcs1));
  strtowcs(wcs1,"Hello World");

  wcsncpy(wcs2, wcs1, 12);
  if (memcmp(wcs2, wcs1, sizeof (wcs1)) == 0)
    printf("PASS\n");
  else
    printf("FAIL\n");

  /*------- wcscmp() test ----------*/
  printf("wcscmp() testing: ");

  /* empty strings */
  strtowcs(wcs1,"");
  strtowcs(wcs2,"");
  if (wcscmp(wcs2,wcs1) == 0)
    printf("PASS");
  else
    printf("FAIL(%d)", wcscmp(wcs2, wcs1));

  strtowcs(wcs2,"H");
  if (wcscmp(wcs1,wcs2) < 0)
    printf(", PASS");
  else
    printf(", FAIL");

  if (wcscmp(wcs2,wcs1) > 0)
    printf(", PASS");
  else
    printf(", FAIL");

  if (wcscmp(wcs2, wcs2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");

  strtowcs(wcs1,"Hello World");
  strtowcs(wcs2,"Hello Worlf");
  if (wcscmp(wcs1, wcs2) < 0)
    printf(", PASS");
  else
    printf(", FAIL");
  
  strtowcs(wcs1,"Hello Worlf2");
  if (wcscmp(wcs1, wcs2) > 0)
    printf(", PASS");
  else
    printf(", FAIL");

  printf("\n");

  /* ------ wcsncmp() -------- */
  printf("wcsncmp() testing: ");

  strtowcs(wcs1, "Hello");
  strtowcs(wcs2, "Heylo");
  if (wcsncmp(wcs1, wcs2 ,0) == 0)
    printf("PASS");
  else
    printf("FAIL");

  if(wcsncmp(wcs1, wcs2,1) == 0)
    printf(", PASS");
  else
    printf(", FAIL");

  if(wcsncmp(wcs1, wcs2,2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");

  if(wcsncmp(wcs1, wcs2, 3) < 0)
    printf(", PASS");
  else
    printf(", FAIL");

  printf("\n");

  /* ------ wcscat() ------- */
  printf("wcscat() testing: ");
  
  strtowcs(wcs1,"");
  strtowcs(wcs2,"Hello World");
  wcscat(wcs1,wcs2);
  if (wcscmp(wcs1, wcs2)==0)
    printf("PASS");
  else
    printf("FAIL");

  strtowcs(wcs1,"");
  wcscpy(wcs3, wcs2);
  wcscat(wcs2, wcs1);
  if (wcscmp(wcs2, wcs3) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  strtowcs(wcs1, "Hello");
  strtowcs(wcs2, "World");
  strtowcs(wcs3, "HelloWorld");
  wcscat(wcs1,wcs2);
  if (wcscmp(wcs1, wcs3) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  printf("\n");

  /* ----- wcsncat() ------ */
  printf("wcsncat() testing: ");
  

  /* 0 character test */
  strtowcs(wcs1,"");
  strtowcs(wcs2,"");
  wcsncat(wcs1,wcs2,1);
  if (wcscmp(wcs1,wcs2) == 0)
    printf ("PASS");
  else
    printf ("FAIL");

  /* 0 buffer test */
  strtowcs(wcs1,"");
  wcsncat(wcs1,wcs2,0);
  if (wcscmp(wcs1,wcs2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");

  /* 0 Buffer, with non-empty destination */
  strtowcs(wcs1,"Hi");
  wcsncat (wcs1,wcs2,0);
  strtowcs(wcs2,"Hi");
  if (wcscmp(wcs1, wcs2)==0)
    printf(", PASS");
  else
    printf(", FAIL");

  /*1 buffer, non empty destination, empty source */
  strtowcs(wcs1, "Hi");
  strtowcs(wcs2, "");
  wcsncat (wcs1, wcs2,1);
  strtowcs(wcs2,"Hi");
  if (wcscmp(wcs1, wcs2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  /*0 Buffer, non empty source and destination */
  strtowcs(wcs1, "Hi");
  strtowcs(wcs2, " There");
  
  wcsncat(wcs1,wcs2,0);
  strtowcs(wcs2,"Hi");
  if (wcscmp(wcs1,wcs2)==0)
    printf(", PASS");
  else
    printf(", FAIL");

  /* 1 buffer, non empty source and destination */
  strtowcs(wcs2, " There");
  wcsncat(wcs1, wcs2, 1);
  strtowcs(wcs2,"Hi ");
  if (wcscmp(wcs1,wcs2) == 0)
    printf (", PASS");
  else
    printf(", FAIL");

  /* 3 buffer, non empty source and destination */
  wcsncat(wcs1, wcs2, 3);
  strtowcs(wcs2,"Hi Hi ");
  if (wcscmp(wcs1, wcs2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");

  /* buffer larger than source, non-empty destination */
  wcsncat(wcs1, wcs2, 200);
  strtowcs(wcs2,"Hi Hi Hi Hi ");
  if (wcscmp(wcs1, wcs2) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  printf("\n");


  /* wcschr() testing */
  printf("wcschr() testing: ");
  strtowcs(wcs1,"AlphaBetaGammaDelta123");
  strtowcs(wcs2,"");
  if ( wcschr(wcs2,chrtowc('a'))==NULL)
    printf("PASS");
  else
    printf("FAIL");

  if ( *(wcschr(wcs2,chrtowc(0))) == 0x00000000)
    printf(", PASS");
  else
    printf(", FAIL");
  if ( *(wcschr(wcs1,chrtowc(0))) == 0x00000000)
    printf(", PASS");
  else
    printf(", FAIL");
  if (wcscmp(wcs1, wcschr(wcs1, chrtowc('A'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  strtowcs(wcs2,"aBetaGammaDelta123");
  if (wcscmp(wcs2, wcschr(wcs1, chrtowc('a'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  strtowcs(wcs2,"3");
  if (wcscmp(wcs2,wcschr(wcs1, chrtowc('3'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  if (wcschr(wcs1, chrtowc('4')) == NULL)
    printf(", PASS");
  else
    printf(", FAIL");
  printf("\n");


  /* wcsrchr() testing */
  printf("wcsrchr() testing: ");
  strtowcs(wcs1,"AlphaBetaGammaDelta123");
  strtowcs(wcs2,"");
  if ( wcsrchr(wcs2,chrtowc('a'))==NULL)
    printf("PASS");
  else
    printf("FAIL");

  if ( *(wcsrchr(wcs2,chrtowc(0))) == 0x00000000)
    printf(", PASS");
  else
    printf(", FAIL");
  if ( *(wcsrchr(wcs1,chrtowc(0))) == 0x00000000)
    printf(", PASS");
  else
    printf(", FAIL");
  if (wcscmp(wcs1, wcsrchr(wcs1, chrtowc('A'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  strtowcs(wcs2,"a123");
  if (wcscmp(wcs2, wcsrchr(wcs1, chrtowc('a'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  strtowcs(wcs2,"3");
  if (wcscmp(wcs2,wcsrchr(wcs1, chrtowc('3'))) == 0)
    printf(", PASS");
  else
    printf(", FAIL");
  if (wcsrchr(wcs1, chrtowc('4')) == NULL)
    printf(", PASS");
  else
    printf(", FAIL");
  printf("\n");

  /* testing wcsspn() */
 

  return 0;
}



