/*
 * Copyright (c) 1999
 *      David Cross <crossd@cs.rpi.edu>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef __wcs_h__
#define __wcs_h__
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned int wint_t;

wchar_t *wcscat(wchar_t *, const wchar_t *);
wchar_t *wscat (wchar_t *, const wchar_t *);

wchar_t *wcsncat(wchar_t *, const wchar_t *, size_t);
wchar_t *wsncat (wchar_t *, const wchar_t *, size_t);

int wcscmp(const wchar_t *, const wchar_t *);
int wscmp (const wchar_t *, const wchar_t *);

int wcsncmp(const wchar_t *, const wchar_t *, size_t);
int wsncmp (const wchar_t *, const wchar_t *, size_t);

wchar_t *wcscpy(wchar_t *, const wchar_t *);
wchar_t *wscpy (wchar_t *, const wchar_t *);

wchar_t *wcsncpy(wchar_t *, const wchar_t *, size_t);
wchar_t *wsncpy (wchar_t *, const wchar_t *, size_t);

size_t wcslen(const wchar_t *);
size_t wslen(const wchar_t *);

wchar_t *wcschr(const wchar_t *, wchar_t);
wchar_t *wschr (const wchar_t *, wchar_t);
wchar_t *windex(const wchar_t *, wchar_t);

wchar_t *wcsrchr(const wchar_t *, wchar_t);
wchar_t *wsrchr(const wchar_t *, wchar_t);
wchar_t *wrindex(const wchar_t *, wchar_t);

wchar_t *wcspbrk(const wchar_t *, const wchar_t *);
wchar_t *wspbrk (const wchar_t *, const wchar_t *);

wchar_t *wcswcs(const wchar_t *, const wchar_t *);

size_t wcsspn(const wchar_t *, const wchar_t *);
size_t wsspn(const wchar_t *, const wchar_t *);

size_t wcscspn(const wchar_t *, const wchar_t *);
size_t wscspn (const wchar_t *, const wchar_t *);

wchar_t *wcstok(wchar_t *, const wchar_t *);
wchar_t *wstok (wchar_t *, const wchar_t *);

wchar_t *wcssep(wchar_t **wcsp, wchar_t *wcdelim);
wchar_t *wssep(wchar_t **wcsp, wchar_t *wcdelim);

wchar_t *strtowcs(wchar_t *, const char *);
char *wcstostr(char *, const wchar_t *);

char wctochr(const wchar_t);
wchar_t chrtowc(const char);

#ifdef __cplusplus
}
#endif

#endif
